function [flag, X] = FourierProjection( tmin , res, Q, Qbar, M , Y,DY,DDY,DDDY)
%FourierProjection  -- This takes a collection of functions and produces 
%                   -- Converts Pointwise bounds to Fourier Bounds.
%                   -- See Algorithm 5.1  --
%                   -- The output is a flag \in { 0,1} and a cube X
%                             if for some reason a contradiction is derived
%                             then we set flag := 1; otherwise flag :=0.
%                             In practice, flag:=1 is very very rare.
%
% The inputs are given as follows 
%     M    -    The (complex) Galerkin projection dimension
%
% Essentially bounds L on the period length:
%     Q    -    Interval bounds on the amount of time the sops is << POSITIVE >>
%     Qbar -    Interval bounds on the amount of time the sops is << NEGATIVE >>
%
% Bounding functions on the derivatives of y.
% Note: This is hard-coded to take s=3.
% See also the paper ``Stability and uniqueness of slowly oscillating periodic solutions to Wrights equation''
%   for further discussion how the bounding functions are stored.  
%
%     Y    -    Interval bounding function for Y
%     DY   -    Interval bounding function for  Y'
%     DDY  -    Interval bounding function for  Y''
%     DDDY -    Interval bounding function for  Y'''
%
%  These are some computational paramters needed for the bounding functions
%     tmin -    The min
%     res  -    The time resolution of the bounding functions




% 1. We calculate the frequency
    
    % The period length of a SOPS is the amount of time the function is
    % positive, plus the amount of time it is negative.
    L = Q + Qbar;
    % The frequency is << 2 pi / L >>
    omega = 2 * intval('pi') / L;

    
    
% 2. In this step we calculate interval bounds for:
%               a_{k,s}^\pm         b_{k,s}^\pm
% 
%   Instead of calculating delta separately, we combine into a single
%   integral. That is
%   This is easily accomplished by multiplying the sin & cos functions by
%   the interval [-1,1] when the time is between [L_min , L_max].


    % We create indicies corresponding to the time: 
    %  L_max - 1/res,        L_min,       0.
    max_index = ceil((sup(L) - tmin)*res);      % --- a  left end point (normally we add +1, but we dont here.)
    min_index = floor((inf(L) - tmin)*res);     % --- a  left end point
    zero_index = -tmin*res +1;                  % --- a  left end point

    
    % We cutoff our bounding functions onto the time interval [0,L_max];
        Y    = Y(zero_index:max_index);
        DY   = DY(zero_index:max_index);
        DDY  = DDY(zero_index:max_index);
        DDDY = DDDY(zero_index:max_index);

        dY = [ Y ; DY ; DDY ; DDDY ];

    % We transpose dY into column vector
        dY = transpose( dY);

    
    % We define the Sine and Cosine functions   
    
        % We create a vector along the left grid points of [0,L_max]
        time = (0: (max_index -zero_index)) /intval(res);
        % We make each left gridpoint into an interval including all time
        % up to the corresponding right grid point. 
        time = time + infsup(0,1)/res;
        

        % We create a blank matrix where each row will correspond to :
        %   omega*k*t  for t \in [ 0 , L_max]
        KT = intval(zeros( M, max_index -zero_index+1)); % -- we start at i=0; add +1.

        % We create a bounding function for k*omega*t for t \in [ 0 , L_max]
        omega_time = omega* time;
        for k = 1 : M 
            KT(k,:) = k*omega_time;
        end

        % This creates a matrix with M rows and L_max*res columns.
        % E.g. the entry COS_{i,j} corresponds to interval bounds for: 
        %       cos( omega i * [ (j-1)*res , j*res ]  )
        COS = cos(KT);
        SIN = sin(KT);
        
        
        % DELTA
        %   -- To account for delta, multiply the sin & cos functions by
        %   -- the interval [-1,1] when the time is between [L_min , L_max].
        
        % We calculate the index corresponding to L_min       
        L_min_index = min_index - zero_index +1;

        % We iterate through times [L_min , L_max]
        for j = L_min_index :  ( max_index -zero_index+1)
            % We multiply each column of COS and SIN by [-1,1]
            COS(:,j) = COS(:,j) * infsup(-1,1);
            SIN(:,j) = SIN(:,j) * infsup(-1,1);
        end

    %   We calculate interval bounds for:
    %    a_{k,s}^\pm         b_{k,s}^\pm
    %       
    %   The integral of the two functions multiplied together is a dot product

    COS_dot_dY = (COS * dY ) ;
    SIN_dot_dY = (SIN * dY) ;
    
    % We multiply by our time step, which is 1/res,
    A = COS_dot_dY / res;
    B = SIN_dot_dY / res;

    % For the matricies A,B 
    %   -- rows     correspond to k / Fourier mode
    %   -- columns  correspond to derivative
    
    
% 3. We define A'_{k,s} , B'_{k,s} 

    % We define constants R_{k,s}, 
    %   which we will divide    <<  a_{k,s} & b_{k,s}  >>   by
    for s = 0:3
        R(:,s+1) = transpose(2*intval('pi')* omega^(s-1) * (1:M).^s);
    end
    
    % We define A'_{k,s} 
        A0_prime = A(:,1) ./ R(:,1);
        A1_prime = A(:,2) ./ R(:,2);
        A2_prime = A(:,3) ./ R(:,3);
        A3_prime = A(:,4) ./ R(:,4);
    % We define B'_{k,s} 
        B0_prime = B(:,1) ./ R(:,1);
        B1_prime = B(:,2) ./ R(:,2);
        B2_prime = B(:,3) ./ R(:,3);
        B3_prime = B(:,4) ./ R(:,4);
    
    
    % We define A_{k,s} 
        A0 =  A0_prime ;
        A1 = -B1_prime ;
        A2 = -A2_prime ;
        A3 =  B3_prime ;
    % We define B_{k,s} 
        B0 = -B0_prime ;
        B1 = -A1_prime ;
        B2 =  B2_prime ;
        B3 =  A3_prime ;
        
    % We define a matrix containing A_{k,s} and B_{k,s} for s=0,1,2,3.
    A = [A0 A1 A2 A3]; 
    B = [B0 B1 B2 B3];

    
    
    
% % % This displays the bounds created for each derivative.   
% % Uncomment for creating Figure 3
%         disp('Y')
%         [A0 B0]
%         disp('DY')
%         [A1 B1]
%         disp('DDY')
%         [A2 B2]
%         disp('DDDY')
%         [A3 B3]


% 4. We take the intersection of our various estimates.
    % We first assume that all intersections will be non-empty.    
    flag =0;
    % If it is empty, then we will reject this cube.
    
    [flag_A12, A12] =emptyintersect(A(:,1),A(:,2));
    [flag_A34, A34] =emptyintersect(A(:,3),A(:,4));
    [flag_A, A_cap] =emptyintersect(A12,A34); 
    if any([flag_A12 ;flag_A34; flag_A])
        disp('Projection-Empty-Intersection')
        flag =1;
    end
    
    [flag_B12, B12] =emptyintersect(B(:,1),B(:,2));
    [flag_B34, B34] =emptyintersect(B(:,3),B(:,4));
    [flag_B, B_cap] =emptyintersect(B12,B34); 
    if any([flag_B12 ;flag_B34; flag_B])
        disp('Projection-Empty-Intersection')
        flag =1;
    end
    
% 5. We calculate the tail estimates -- for S = 3.
    % We calculate the fourier coefficients of middle function
    A_mid = intval(A(:,4).mid);
    B_mid = intval(B(:,4).mid);
    
    % We calculate a relatively thin bounding for y^S_M
    DDDY_mid =            2*( omega^3 * transpose(A_mid).* ((1:M).^3 )) * sin(KT) ;
    DDDY_mid = DDDY_mid + 2*( omega^3 * transpose(B_mid).* ((1:M).^3) ) * cos(KT);    
 
    % We calculate the L^1 norm
    C0 = sum(  abs((DDDY-DDDY_mid)))/res;
    % We divide through by  << 2 pi omega^S-1 >>
    C0 = C0 /  ( 2 * intval('pi') * (omega^2) );    

    
% 6. We define the output X    (without alpha)
    X = Cube('null',omega,A_cap,B_cap,M) ;
    X.C0 = intval(C0.sup);
    X.s = 3;

    

end
