function [ Answer_X, Combined_Reject, success ,I_A, I_B] = PostVerify( Answer_X,  reject_X, bif_X, prune_rep)
%PostVerify -- This recomines the rejected cubes, verifies them, and then checks that there is a unique SOPS for all alpha. 
%           -- See Algorithm 6.3 -- 

% OUTPUTS 
%   Answer_X            --  This the resulting list of cubes: \mathcal{A} 
%   Combined_Reject     --  This the resulting list of cubes: \mathcal{R} 
%   success             --  A boolean output reflecting whether the algorithm
%                               reaches Step 10
%   I_A                 --  The alpha-projection of \cup \mathcal{A} 
%   I_B                 --  The alpha-projection of \cup \mathcal{B} 

% INPUTS 
%   Answer_X    --  The set \mathcal{A} output from Alg. 6.1
%   reject_X    --  The set \mathcal{R} output from Alg. 6.1
%   bif_X       --  The set \mathcal{B} output from Alg. 6.1
%   prune_rep   --  The  variable n, of how many times we repeatedly prune

    % We assume we do not succeed
    success=0;



    % 1. We go through reject_X and 
    %    combine everything that isn't a pointwise intersection
    % 2. Then we subdivide cube along alpha
    
    Combined_Reject = {};    
    % We go through the "upper-diagonal half" of the set product 
    %       reject_X \times reject_X 
    % looking for cubes which nontrivially intersect along alpha 
    while isempty(reject_X) == 0
        % We take a cube X_loc from reject_X, and look for other cubes
        %   of lower index which nontrivially intersect along alpha.
        % Note: Since we then form a new cube which contains their union, 
        %       we make a copy, and not alter the original cube.
        X_loc = copy(reject_X{end});
        reject_X(end) =[];

        % We find everything that intersects with X_loc and remove it
        %   starting at the end of the list, and working our way down        
        k=length(reject_X);
        while k >0 
            Y = copy(reject_X{k});
            % Check to see if there is a non-trivial intersection
            [empty_a,  Ia_local] = emptyintersect(X_loc.alpha,Y.alpha);
            % There is an intersectional which is more than a point.
            if (~empty_a )&& (Ia_local.rad > 0)
                % We add the new thing that intersects I_a
                X_loc.alpha = hull(X_loc.alpha,Y.alpha);
                X_loc.omega = hull(X_loc.omega,Y.omega);
                X_loc.A = hull(X_loc.A,Y.A);
                X_loc.B = hull(X_loc.B,Y.B);
                X_loc.C0 = hull(X_loc.C0,Y.C0);
                % We remove Y from reject_X: Since Y is contained in X_loc, 
                %    and thereby Combined_Reject, we do not lost any solutions
                reject_X(k) = [];
            end
            k = k-1;
        end% Cubes of lower index than X_loc.
        
        % We add the intersected thing to the end of Combined_Reject
        
        % 2.  We split alpha into 2.
        X_loc1 =copy(X_loc);
        X_loc2 =copy(X_loc);
        
        X_loc1.alpha = infsup(inf(X_loc.alpha),mid(X_loc.alpha));
        X_loc2.alpha = infsup(mid(X_loc.alpha),sup(X_loc.alpha));
        Combined_Reject{end+1} = X_loc1;
        Combined_Reject{end+1} = X_loc2;
    end% Steps 1&2
    
    disp('Combined Reject')
    % 3. We prune everything in the list several times. 
    %    We decrease the index, because this works better if we remove a cube
    k = length(Combined_Reject);
    while k >0
        % We prune the cube 'prune_rep' number of times, 
        %  stopping early if possible.
        for i = 1:prune_rep
            flag = Prune(Combined_Reject{k});
            if flag ==1 
                % Cube can't contain solutions- discard
                Combined_Reject(k)=[];
                break%--Stop-Pruning
            elseif flag ==2 
                % Cube is in the bifurcation neighborhood
                % This would be unusual, but not bad.
                disp('Add to bif region')
                bif_X{end+1} = Combined_Reject{k};
                Combined_Reject(k)=[];
                break%--Stop-Pruning
            elseif flag ==3
                % Krawczyk maps cube inside itself
                Answer_X{end+1} = Combined_Reject{k};
                Combined_Reject(k)=[];
                break%--Stop-Pruning
            end
            % If flag == 0 we prune again
        end% pruning
        k=k-1;
    end% checking cubes in Combined_Reject
    
    % 4. If there are still rejected cubes, we exit the algorithm.
    if ~isempty(Combined_Reject)
        disp('There are still rejected cubes')
        % exit the algorithm
        I_A = 'null';
        I_B = 'null';
        return
    else
        disp('No rejected Cubes!!!')
        
        
        % 5a. We create the interval I_A 
        if isempty(Answer_X)
            I_A = 'null';
        else
            I_A = Answer_X{1}.alpha;
            for k = 1:length(Answer_X)
                I_A = hull(I_A,Answer_X{k}.alpha);
            end
        end
        % 5b. We create the interval I_B 
        if isempty(bif_X)
            I_B = 'null';
        else
            I_B = bif_X{1}.alpha;
            for k = 1:length(bif_X)
                I_B = hull(I_B,bif_X{k}.alpha);
            end
        end
 
        % 6 We create \mathcal{I}_B'
        Intersect_B_prime = {};       

        %if \mathcal{B} is empty, then we don't need to do any work.
        if ~isempty(bif_X)
            disp('Constructing Intersect B')
            % We go through all the cubes in \mathcal{A}
            for k = 1 : length(Answer_X)
                X_loc = Answer_X{k};
                % We check if it intersects with I_B
                [empty_a,  Ia_local] = emptyintersect(X_loc.alpha,I_B);
                if (~empty_a )
                    % In this case, X_loc is in Intersect_B   (no prime) 
                    
                    % We change alpha to the interseted value 
                    Y = copy(X_loc);
                    Y.alpha =Ia_local;
                    % We add nonempty intersections to the list
                    Intersect_B_prime{end+1} = Y;
                end
            end

            % 7. We check to see if each thing in Intersect_B is verified.
            disp('Verifying Intersect B')
            % We prune everything in the list several times. 
            k = length(Intersect_B_prime);
            while k >0
                for i = 1:prune_rep 
                    flag = Prune(Intersect_B_prime{k});
                    % We remove cubes from Intersect_B_prime only if flag =2
                    if flag ==2
                        Intersect_B_prime(k)=[];
                        break
                    end
                end
                k=k-1;
            end      
            
        end% \mathcal{B} being non-empty
        
        

        % 8 (a) We create \mathcal{I}_A'
        Intersect_A_prime = {};
        %if \mathcal{A} is empty, then we don't need to do any work.
        if ~isempty(Answer_X)
            
            % We go through and cover parts of cubes which intersect each
            % other in the alpha-dimension.
            for k = 1 : length(Answer_X)
                X_loc = Answer_X{k};
                % Due to symmetry, we only need to check the "upper
                % diagonal half" of the cubes
                for i = k+1:length(Answer_X)
                    % We check for intersections 
                    [empty_a,  Ia_local] = emptyintersect(X_loc.alpha,Answer_X{i}.alpha);
                    % If this is nonempty, then the pair of cubes is in
                    % \mathcal{I}_A  (no prime).
                    if (~empty_a )
                        % We make the cube Z (stored as Y) 
                        Y = copy(Answer_X{i});
                        % Z's alpha is the intersection of the alpha components 
                        Y.alpha =Ia_local;
                        % Everything else about Z is the union of X and Y.
                        Y.omega = hull(X_loc.omega,Y.omega);
                        Y.A = hull(X_loc.A,Y.A);
                        Y.B = hull(X_loc.B,Y.B);
                        Y.C0 = hull(X_loc.C0,Y.C0);
                        % We add nonempty intersections to the list
                        Intersect_A_prime{end+1} = Y;
                    end% Y intersects with X
                end% cubes Y
            end% cubes X



            % 9. We check to see if each thing in Intersect_A is verified.
            disp('Verifying Intersect A')

            % We prune everything in the list several times. 
            k = length(Intersect_A_prime);
            while k >0
                for i = 1:prune_rep 
                    flag = Prune(Intersect_A_prime{k});
                    % We remove cubes from Intersect_A_prime only if flag=3
                    if flag ==3 
                        Intersect_A_prime(k)=[];
                        break
                    end
                end
                k=k-1;
            end
        end
     
        
        % 10. If we reach this point and we have removed everything from
        % the lists, then we are successful.
        

        
        if isempty(Intersect_A_prime) && isempty(Intersect_B_prime) 
            success =1;
            disp('Success!!!!  Success!!!!  Success!!!!  Success!!!!')
            disp('Success!!!!  Success!!!!  Success!!!!  Success!!!!')
            disp('Success!!!!  Success!!!!  Success!!!!  Success!!!!')        
            
        end
    end

end

