function [ List_Y_I, List_DY_I ,List_DDY_I, List_DDDY_I ] = X2Y( res, alpha, List_X_I)
%X2Y    -- This is a part of Algorithm 5.7 (Steps 2&3)
%       -- (Step 2) Converts Wright's Equation from the X-representation to the Y-representation
%       -- (Step 3) Calculates the higher order derivatives (s=0,1,2,3). 


% The bounding functions X_I are stored for times [ t_min, t_max];
% The paper ``Stability and uniqueness of slowly oscillating periodic solutions to Wrights equation''
%   always uses tmin = -4.
tmin=-4;
% Each bounding function is represented by a vector with interval values. 
% Each component of the vector corresponds to a time interval of width
% 1/res.


%       In order to calculate bounds on the derivative y^(s)(t_0), 
%       we need to have bounds on y^(r)(t_0) and y^(r)(t_0-1)  
%                                                         0 <= r <= s-1
%       Hence, we create bounding functions for 
%       y(t-0)          &       y(t-1)
%       y'(t-0)         &       y'(t-1)
%       y''(t-0)        &       y''(t-1)
%       y'''(t-0)       &       
    
% We represent the functions y^(r)(t-1) by essentially,
%   taking the function y^(r)(t-1), shifting it to the right one second,
%   and defining the function as zero for       t \in [tmin,tmin+1]

% This is necessary for the bounding functions to all have the same size as
%   vectors.
% Here we create a bounding function which is zero for one time unit (a second)
zero_second = intval(zeros(1,res));

% We create lists to store the bounding functions for Y^s,  0 <= s <= 3
List_Y_I    = List_X_I;
List_DY_I   = List_X_I;
List_DDY_I  = List_X_I;
List_DDDY_I = List_X_I;


for k =1 : length(List_X_I)   
    % We switch from the exponential to the quadratic version
    % We create y(t-0)
    Y0= exp(List_X_I{k})-1;    
    
    % We determine the maximum time/index of our bounding function
    length_Y = length(Y0);
    
    % We create y(t-1)
    Y1 = [zero_second  Y0(1:length_Y -res)];    
    
    % We create the first derivative
    % y'(t) = - \alpha y(t-1)  [1 + y(t)]    
    DY0 = - alpha * Y1 .* (1+Y0);
    
    % We define y'(t-1)
    DY1 = [zero_second  DY0(1:length_Y -res) ]; 

     
%     y''(t) &=& -\alpha  [
%                             y'(t-1) [1 + y(t)] + y(t-1) y'(t)
%                         ]

    DDY0 = - alpha * (  DY1 .* (1 + Y0 ) + Y1 .* DY0) ;
    
    % We define y''(t-1)
    DDY1 = [zero_second  DDY0(1:length_Y -res) ]; 

    
    % y'''(t)  =  - \alpha  [ 
    %                           y''(t-1) [1+y(t) ] + 2 y'(t-1)y'(t) + y(t-1) y''(t) 
    %                       ]
    DDDY0 = - alpha *(  DDY1 .* (1 + Y0) + 2 *DY1.*DY0 + Y1 .* DDY0 ) ;
    
    
    List_Y_I{k} =Y0;
    List_DY_I{k} =DY0;
    List_DDY_I{k} =DDY0;
    List_DDDY_I{k} =DDDY0;

end

end

